	subroutine derivs(znow,yp,grad)

C	SUBROUTINE TO CALCULATE PRESSURE GRADIENT BETWEEN THE PREVIOUS
C	AND THE CURRENT DEPTH.

C	DECLARE VARIABLES USED IN THIS SUBROUTINE

	implicit none
	real*8 area(10000),z(10000)
	real*8 beta,blkgas,blkmix,bubnum,bubrad,bubvol,Ca,dadz,
     *		diam,dfirst,dfinal,dh2o,dvdrmax,
     *		dvdth2o,dvdz,edot,emax,eta,f,fo,g,grad,m,
     *		mdot,p,rey,rhomix,sv,term1,v,vex,vfgas,
     *		vm,vxtls,xsarea,
     *		yp,zfinal,znow
	integer i,icalc,ifrag

C	DECLARE VARIABLES LISTED IN THE COMMON BLOCK BUT NOT USED
C	IN THIS SUBROUTINE

	real*8 mach(10000),pres(10000),re(10000),rho(10000),
     *		vel(10000),vesic(10000),visc(10000),wtoxan(9),
     *		xcomp(10)
      real*8 blkmag,cpm,cp,cv,dgdxw,dhdp,dpdz,dvmdt,dz,
     *		enthalpy,entropy,eps,etagas,etamag,exmol,gamma,gwm,
     *		h,hf,hwm,mg,mh2o,mm,msil,mxtls,mw,
     *		mwttot,pfinal,
     *		r,rhof,rhom,rhoxtls,
     *		s,swm,temp,xtls
	integer iend,ifinal,im,ip,io,iruns,itherm,ives
	real*8  cpH2O,cvH2O,gH2O,hH2O,sH2O,vH2O


C	COMMON BLOCKS

      Common/block1/area,mach,pres,re,
     *		rho,vel,vesic,visc,z
      Common/block2/beta,bubnum,Ca,cp,cv,
     *		dadz,dfinal,dfirst,dhdp,diam,dpdz,dvdz,dz,
     *		emax,eps,eta,etagas,etamag,
     *		exmol,f,fo,g,gamma,h,hf,m,mdot,mg,mh2o,mm,
     *		msil,mw,mxtls,pfinal,r,rey,rhof,rhomix,
     *		rhoxtls,s,
     *		sv,temp,v,vex,vfgas,vm,vxtls,xsarea,xtls,zfinal
 	Common/ints/i,icalc,iend,ifinal,ifrag,im,ip,io,
     *		iruns,itherm,ives
 	common/melt/blkmag,cpm,dh2o,enthalpy,entropy,mwttot,rhom,wtoxan
 	common/water/cpH2O,cvH2O,gH2O,hH2O,sH2O,vH2O,blkgas

	   if (icalc.eq.1) then
	     p = yp
	     diam = dfirst + (dfinal-dfirst)*(znow-z(1))/(zfinal-z(1))
	     xsarea = 3.14159 * diam**2/4.
	    else
	     xsarea = yp
	     if (xsarea.lt.0.)  xsarea = 0.0001
	     diam = 2.*sqrt(xsarea/3.14159)
	     p = pfinal + dpdz*(znow-zfinal)
	   end if

C	DETERMINE NEW MASS FRACTION GAS

	if ((ifrag.eq.1).and.(ives.eq.2)) go to 1265
	if (p.ge.0.) then
		dh2o = .01*s*(p/1.0d+06)**beta
	    if (dh2o.le.0.) dh2o = 1.0d-03
		mm = msil/(1.-dh2o)
		mg = 1.-mxtls-mm
	    if (mg.lt.0.) then
			mg=0.
			mm = 1.-mxtls
			dh2o = mw/mm
	    end if
	  else
	    dh2o = 1.0d-03
		mg = mw*(1.-dh2o)/msil
		mm = 1.-mg-mxtls
	end if
1265	continue

C	DETERMINE PROPERTIES OF MAGMA AND GAS

	if (p.gt.0.) then
	  call multisi(temp,p/1.0d+05,blkmag,cpm,dh2o,enthalpy,
     *		  entropy,mwttot,rhom,wtoxan,xcomp,hwm,swm,gwm,dgdxw,
     *		  dvmdt)
	  call haarsi(temp,p/1.0d+05,cpH2O,cvH2O,gH2O,hH2O,sH2O,
     *		vH2O,blkgas,dvdth2o)
	end if

	cp	= cpH2O/mh2o
	cv	= cvH2O/mh2o
	rhof = (1.0D+06*.0180152)/vH2O
	gamma = cp/cv

C	CALCULATE MIXTURE DENSITY, VFGAS
	rhomix = 1./(mm/rhom + mg/rhof + mxtls/rhoxtls)
	vfgas = mg*rhomix/rhof 
	if (vfgas.gt.1.0) vfgas = 0.99999d+00
	vm = mm*rhomix/rhom
	vxtls = mxtls*rhomix/rhoxtls

C	CALCULATE NEW VELOCITY, VELOCITY GRADIENTS, STRAIN RATES
	v = mdot/(rhomix*xsarea)
	if ((i.gt.1).and.(znow.gt.z(i))) then
		dvdz = (v-vel(i))/(znow-z(i))	!extensional strain rate in z direction.
	  else if (i.gt.1) then
		dvdz = (v-vel(i-1))/(znow-z(i-1))
	  else
		dvdz = 0.
	end if

	if (ifrag.eq.0) then				!shear-strain rate at conduit wall
		dvdrmax = -2.*v/diam
	  else
		dvdrmax = 0.
	end if
	emax = dvdz/2. + dsqrt((dvdz/2.)**2 + dvdrmax**2)	!max strain rate

C	CALCULATE BULK MODULUS AND SONIC VELOCITY
	blkmix = 1./(vfgas/blkgas + (1.-vfgas)*(1.-0.01*xtls)/blkmag + 
     *		(1.-vfgas)*0.01*xtls/1.0d+11)
	if (blkmix.lt.0.) pause 'blkmix < 0'
      sv = dsqrt(blkmix/rhomix)
	  
C	CALCULATE VISCOSITY
	if (ifrag.eq.0) then
		call vis
C		CALCULATE CAPILLARY NUMBER, VEX
		if (vfgas.gt.0.) then				
			edot = 4.*v/(3.*diam)
			bubvol = vfgas/(bubnum*vm)
			bubrad = 10.**((dlog10(3.*bubvol/(4.*3.14159)))/3.)
			ca = etamag * edot * bubrad / 3.4d-01
			vex = (2./3.14159) * datan(5.*dlog10(ca))
		  else
			Ca = 1.
			vex = 0.
		end if
		eta = etamag*(1.-vfgas)**vex
	 else
		if (vfgas.gt.0.40) then
		  etagas = (5.3d-05 * (1.-(1.-vfgas)/0.62)**(-1.5))
		 else
		  etagas = 5.3d-05
	    end if
		eta = etagas
	    Ca = 1.
	end if

C	CALCULATE REYNOLDS, NUMBER, FRICTION FACTOR
	rey = rhomix*v*diam/eta
	f = 16./rey + fo

C	CALCULATE MACH NUMBER

	m = v/sv

C	CALCULATE GRADIENT IN PRESSURE OR XSAREA

	if (icalc.eq.1) then
	   term1 = 2.*f*v**2/diam
	   grad = (rhomix*(-g - 2.*f*v**2/diam + v**2/xsarea * dadz))/
     *		(1.-m**2)
	   dpdz = grad
	  else
	  	   grad = xsarea/v**2 * ( dpdz*(1.-m**2)/rhomix + g +
     *		(2.*f*v**2)/diam )
	   dadz = grad
	end if

	return
	end
